#!/bin/sh

set -e

[ -z "$1" ] && echo "usage: $0 OPENSSL_CNF [OUTPUT_DIR]" && exit 2

openssl_cnf="$1"
[ -n "$2" ] && outdir="$2" || outdir="."

# generate a certificate authority key, store only in memory
rootca=$("openssl" "genrsa" "2048")

# self-sign rootca, store cert as file
echo "$rootca" | "openssl" "req" "-x509" "-new" "-nodes" "-sha512" "-days" "3650" "-config" "$openssl_cnf" "-key" "/dev/stdin" "-out" "$outdir/ditm-127.0.0.1-ca.crt" "-subj" "/C=US/O=Splintermail Localhost Authority/OU=Org/CN=splintermail.localhost"

# create key
"openssl" "genrsa" "-out" "$outdir/ditm-127.0.0.1-key.pem" "2048"

# create certificate sign request
"openssl" "req" "-new" "-config" "$openssl_cnf" "-key" "$outdir/ditm-127.0.0.1-key.pem" "-out" "$outdir/sig_req.csr" "-subj" "/C=US/O=Splintermail on localhost (127.0.0.1)/OU=Org/CN=127.0.0.1"

# sign with rootca
echo "$rootca" | "openssl" "x509" "-req" "-days" "3650" "-sha512" "-extensions" "req_ext" "-CAcreateserial" "-extfile" "$openssl_cnf" "-in" "$outdir/sig_req.csr" "-CA" "$outdir/ditm-127.0.0.1-ca.crt" "-CAkey" "/dev/stdin" "-out" "$outdir/ditm-127.0.0.1-cert.pem"

# turn the certificate into a proper chain
cat "$outdir/ditm-127.0.0.1-ca.crt" >> "$outdir/ditm-127.0.0.1-cert.pem"

# cleanup unecessary files
rm -f "$outdir/$(echo ditm-127.0.0.1-ca.crt | sed -e 's/\..*//').srl"
rm -f "$outdir/sig_req.csr"

unset rootca
